package com.sinoqixin.bgdrocketmq.config;

import org.apache.rocketmq.client.consumer.DefaultMQPushConsumer;
import org.apache.rocketmq.common.consumer.ConsumeFromWhere;
import org.apache.rocketmq.remoting.protocol.heartbeat.MessageModel;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

@Configuration
public class MqConsumerBeanConfig {

    @Value("${consumer.namesrvAddr}")
    private String namesrvAddr;

    @Value("${consumer.groupName}")
    private String groupName;

    @Bean(name = "rocketMqClient")
    public DefaultMQPushConsumer initRocketMqClient() {
        DefaultMQPushConsumer consumer = new DefaultMQPushConsumer(groupName);
        consumer.setNamesrvAddr(namesrvAddr);
        // 消费者线程数据量
        consumer.setConsumeThreadMin(20);
        consumer.setConsumeThreadMax(20);
        // 设置一次消费的条数，默认1
        consumer.setConsumeMessageBatchMaxSize(10);
        //消费模式，CLUSTERING：集群消费模式，广播消费模式：BROADCASTING
        consumer.setMessageModel(MessageModel.CLUSTERING);
        // 设置consumer第一次启动时消费位置
        // CONSUME_FROM_FIRST_OFFSET：从第一个偏移量开始消费，即从头开始消费队列中的所有消息
        // CONSUME_FROM_LAST_OFFSET：从最后一个偏移量开始消费，即只消费新到达的消息。这对于只关注最新消息的消费者很有用
        // CONSUME_FROM_TIMESTAMP：从指定的时间戳开始消费。你需要提供一个时间戳，消费者将从该时间戳之后的第一条消息开始消费
        consumer.setConsumeFromWhere(ConsumeFromWhere.CONSUME_FROM_FIRST_OFFSET);

        return consumer;
    }
}
