package com.sinoqixin.bgdrocketmq.dao;


import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;

@Repository
@Slf4j
public class ProducePlanDao {

    @Autowired
    private JdbcTemplate jdbcTemplate;

    /**
     * 更新 电网管理平台生产计划工单表  接口查询字段
     *
     * @param args
     */
    public void updateDwglProducePlanWorkOrderFull(List<Object[]> args) {
        String sql = "MERGE INTO ITGBGD.DWGL_PRODUCE_PLAN_WORK_ORDER AA " +
                "USING ( SELECT ? PROD_PLAN_ID FROM DUAL ) BB ON BB.PROD_PLAN_ID = AA.PROD_PLAN_ID  " +
                "WHEN MATCHED THEN   " +
                "update set PLAN_CODE=?,PLAN_STATE=?,SPECIALITY=?,BUREAU_CODE=?,POWER_GRUD_FLAG=?,WORK_TYPE=?," +
                "PLAN_BEGIN_TIME=?,PLAN_END_TIME=?,ID=?,BUSINESS_TYPE=?,PATROL_CLASSIFY=?,BGD_UPDATE_TIME=sysdate()," +
                "PLAN_TYPE=?,PLAN_NATURE=?,WORK_TEAM_OID=?,WORK_TEAM_OCODE=?,WORK_TEAM_ONAME=?,WORK_SITE_NAMES=?," +
                "WORKING=?,WORK_SMALL_TYPE=?,CREATOR_UID=?,CREATOR_UNAME=?,CREATE_TIME=?,WORK_MASTER_UID=?,WORK_MASTER_UNAME=?," +
                "WORK_MEMBER_UNAMES=?,REAL_BEGIN_TIME=?,REAL_END_TIME=?,BASE_VOLTAGE_ID=?,WORK_DEPT_OID=?,WORK_DEPT_ONAME=?," +
                "PATROL_MODE=?,PROVINCE_CODE=?,FLOW_STATE=?,MONITOR_OPINION=?,AFFIRM_UID=?,AFFIRM_UNAME=?,AFFIRM_TIME=?," +
                "IS_BEQUEATH_QUESTION=?,RISK_LEVEL=?,NOT_APPLY_OUTAGE=?,IMPLEMENT_MODE=?,UPDATE_TIME=?,DATA_FROM=?," +
                "START_WORK_TIME=?,PLAN_TASK_TYPE=?,JOB_CONTENT=?,WORK_RESULT=?,BEQUEATH_QUESTION=?,PLAN_SITE_LST=?,TASK_ID=? " +
                "WHEN NOT MATCHED THEN   " +
                "INSERT " +
                "(PROD_PLAN_ID,PLAN_CODE,PLAN_STATE,SPECIALITY,BUREAU_CODE,POWER_GRUD_FLAG,WORK_TYPE," +
                "PLAN_BEGIN_TIME,PLAN_END_TIME,ID,BUSINESS_TYPE,PATROL_CLASSIFY,BGD_UPDATE_TIME," +
                "PLAN_TYPE,PLAN_NATURE,WORK_TEAM_OID,WORK_TEAM_OCODE,WORK_TEAM_ONAME,WORK_SITE_NAMES," +
                "WORKING,WORK_SMALL_TYPE,CREATOR_UID,CREATOR_UNAME,CREATE_TIME,WORK_MASTER_UID,WORK_MASTER_UNAME," +
                "WORK_MEMBER_UNAMES,REAL_BEGIN_TIME,REAL_END_TIME,BASE_VOLTAGE_ID,WORK_DEPT_OID,WORK_DEPT_ONAME," +
                "PATROL_MODE,PROVINCE_CODE,FLOW_STATE,MONITOR_OPINION,AFFIRM_UID,AFFIRM_UNAME,AFFIRM_TIME," +
                "IS_BEQUEATH_QUESTION,RISK_LEVEL,NOT_APPLY_OUTAGE,IMPLEMENT_MODE,UPDATE_TIME,DATA_FROM," +
                "START_WORK_TIME,PLAN_TASK_TYPE,JOB_CONTENT,WORK_RESULT,BEQUEATH_QUESTION,PLAN_SITE_LST,TASK_ID) " +
                "VALUES(?,?,?,?,?, ?,?,?,?,?, ?,?,sysdate(),?,?, ?,?,?,?,?, ?,?,?,?,?, ?,?,?,?,?, ?,?,?,?,?, ?,?,?,?,?, ?,?,?,?,?, ?,?,?,?,?, ?,?)";
        jdbcTemplate.batchUpdate(sql, args);
    }

    /**
     * 记录 电网管理平台推送过来生产计划工单 接口查询字段
     *
     * @param args
     */
    public void insertDwglProducePlanWorkOrderFullRecord(List<Object[]> args) {
        try {
            List<Object[]> params = new ArrayList<>();
            for (Object[] arg : args) {
                Object[] item = new Object[51];
                System.arraycopy(arg, 0, item, 0, item.length);
                params.add(item);
            }
            String sql = "INSERT INTO ITGBGD.DWGL_PRODUCE_PLAN_WORK_ORDER_RECORD " +
                    "(PROD_PLAN_ID,PLAN_CODE,PLAN_STATE,SPECIALITY,BUREAU_CODE,POWER_GRUD_FLAG,WORK_TYPE, " +
                    "PLAN_BEGIN_TIME,PLAN_END_TIME,ID,BUSINESS_TYPE,PATROL_CLASSIFY,BGD_UPDATE_TIME, " +
                    "PLAN_TYPE,PLAN_NATURE,WORK_TEAM_OID,WORK_TEAM_OCODE,WORK_TEAM_ONAME,WORK_SITE_NAMES, " +
                    "WORKING,WORK_SMALL_TYPE,CREATOR_UID,CREATOR_UNAME,CREATE_TIME,WORK_MASTER_UID,WORK_MASTER_UNAME, " +
                    "WORK_MEMBER_UNAMES,REAL_BEGIN_TIME,REAL_END_TIME,BASE_VOLTAGE_ID,WORK_DEPT_OID,WORK_DEPT_ONAME, " +
                    "PATROL_MODE,PROVINCE_CODE,FLOW_STATE,MONITOR_OPINION,AFFIRM_UID,AFFIRM_UNAME,AFFIRM_TIME, " +
                    "IS_BEQUEATH_QUESTION,RISK_LEVEL,NOT_APPLY_OUTAGE,IMPLEMENT_MODE,UPDATE_TIME,DATA_FROM, " +
                    "START_WORK_TIME,PLAN_TASK_TYPE,JOB_CONTENT,WORK_RESULT,BEQUEATH_QUESTION,PLAN_SITE_LST,TASK_ID) " +
                    "VALUES(?,?,?,?,?, ?,?,?,?,?, ?,?,sysdate(),?,?, ?,?,?,?,?, ?,?,?,?,?, ?,?,?,?,?, ?,?,?,?,?, ?,?,?,?,?, ?,?,?,?,?, ?,?,?,?,?, ?,?)";
            jdbcTemplate.batchUpdate(sql, params);
        } catch (DataAccessException e) {
            e.printStackTrace();
            log.error("记录 电网管理平台推送过来生产计划工单【接口查询字段】 异常，异常信息：{}", e.getMessage());
        }
    }

    /**
     * 更新 电网管理平台生产计划工单表  rocketMq推送字段
     *
     * @param args
     */
    public void updateDwglProducePlanWorkOrder(List<Object[]> args) {
        String sql = "MERGE INTO ITGBGD.DWGL_PRODUCE_PLAN_WORK_ORDER AA " +
                "USING ( SELECT ? PROD_PLAN_ID FROM DUAL ) BB ON BB.PROD_PLAN_ID = AA.PROD_PLAN_ID  " +
                "WHEN MATCHED THEN   " +
                "update set PLAN_CODE=?,PLAN_STATE=?,SPECIALITY=?,BUREAU_CODE=?,POWER_GRUD_FLAG=?,WORK_TYPE=?," +
                "PLAN_BEGIN_TIME=?,PLAN_END_TIME=?,ID=?,BUSINESS_TYPE=?,PATROL_CLASSIFY=?,BGD_UPDATE_TIME=sysdate() " +
                "WHEN NOT MATCHED THEN   " +
                "INSERT " +
                "(PROD_PLAN_ID,PLAN_CODE,PLAN_STATE,SPECIALITY,BUREAU_CODE,POWER_GRUD_FLAG,WORK_TYPE," +
                "PLAN_BEGIN_TIME,PLAN_END_TIME,ID,BUSINESS_TYPE,PATROL_CLASSIFY,BGD_UPDATE_TIME) " +
                "VALUES(?,?,?,?,?, ?,?,?,?,?, ?,?,sysdate()) ";
        jdbcTemplate.batchUpdate(sql, args);
    }

    /**
     * 记录 电网管理平台推送过来生产计划工单 rocketMq推送字段
     *
     * @param args
     */
    public void insertDwglProducePlanWorkOrderRecord(List<Object[]> args) {
        try {
            List<Object[]> params = new ArrayList<>();
            for (Object[] arg : args) {
                Object[] item = new Object[12];
                System.arraycopy(arg, 0, item, 0, item.length);
                params.add(item);
            }
            String sql = "INSERT INTO ITGBGD.DWGL_PRODUCE_PLAN_WORK_ORDER_RECORD  " +
                    "(PROD_PLAN_ID,PLAN_CODE,PLAN_STATE,SPECIALITY,BUREAU_CODE,POWER_GRUD_FLAG,WORK_TYPE," +
                    "PLAN_BEGIN_TIME,PLAN_END_TIME,ID,BUSINESS_TYPE,PATROL_CLASSIFY,BGD_UPDATE_TIME) " +
                    "VALUES(?,?,?,?,?, ?,?,?,?,?, ?,?,sysdate()) ";
            jdbcTemplate.batchUpdate(sql, params);
        } catch (DataAccessException e) {
            e.printStackTrace();
            log.error("记录 电网管理平台推送过来生产计划工单【rocketMq推送】 异常，异常信息：{}", e.getMessage());
        }
    }


    /**
     * 更新 保供电任务信息表
     *
     * @param args
     */
    public void updateBgdTaskInfo(List<Object[]> args) {
        String sql = "MERGE INTO ITGBGD.ECM_EMYA_BGD_TASK_INFO AA " +
                "USING ( SELECT ? TASK_POOL_ID FROM DUAL ) BB ON BB.TASK_POOL_ID = AA.TASK_POOL_ID  " +
                "WHEN MATCHED THEN   " +
                "update set PLAN_ID=?,PLAN_NO=?,PLAN_STATUS=? ";
        jdbcTemplate.batchUpdate(sql, args);
    }

}
