package com.sinoqixin.bgdrocketmq.service;

import cn.hutool.http.HttpException;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson.JSONObject;
import com.sinoqixin.bgdrocketmq.utils.Utils;
import com.sinoqixin.bgdrocketmq.vo.AjaxResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

@Service
@Slf4j
public class GetProducePlanDetailApiService {
    //分省标识
    private final static String TOP_PROVINCE_FLAG = "7F079D562F9A8360BE8CA1292A27AF28";
    private final static String BUSINESS_TYPES = "Source_164";

    @Value("${consumer.queryPlanInfoForPowerSupplyUrl_formal}")
    private String queryPlanInfoForPowerSupplyUrl;
    @Value("${consumer.queryPlanBusinessInfoUrl_formal}")
    private String queryPlanBusinessInfoUrl;
    @Value("${consumer.getToken_url}")
    private String getTokenUrl;
    @Value("${consumer.getToken_account}")
    private String getTokenAccount;
    @Value("${consumer.getToken_password}")
    private String getTokenPassword;
    @Value("${consumer.getToken_authCode}")
    private String getTokenAuthCode;

    public AjaxResult getPlanBusinessInfoByApi(String planIds, String bureauCode, int timeout) {
        if (StringUtils.isBlank(bureauCode)) bureauCode = "0301";
        Map<String, Object> bodyMap = new HashMap<>();
        bodyMap.put("bureauCode", bureauCode);
        bodyMap.put("planIds", planIds);
        bodyMap.put("businessTypes", BUSINESS_TYPES);

        AjaxResult getTokenResult = getToken(timeout);
        String token = StringUtils.EMPTY;
        //无论token获取成功或失败，都进行详情获取
        if (getTokenResult.isSuccess()) {
            token = Utils.getString(getTokenResult.get(AjaxResult.DATA_TAG));
        }
        return queryPlanBusinessInfo(bodyMap, timeout, token);
    }


    public AjaxResult queryPlanBusinessInfo(Map<String, Object> bodyMap, int timeout, String token) {
        Map<String, String> headerMap = new HashMap<>();
        headerMap.put("Content-Type", "application/json");
        headerMap.put("access-token", token);
        headerMap.put("topProvinceFlag", TOP_PROVINCE_FLAG);

        log.info("【queryPlanBusinessInfo】开始请求，地址：{}，请求头：{}，请求体：{}", queryPlanBusinessInfoUrl, headerMap, bodyMap);
        try(HttpResponse httpResponse = HttpRequest.post(queryPlanBusinessInfoUrl)
                .headerMap(headerMap, false)
                .timeout(timeout)
                .body(JSONObject.toJSONString(bodyMap))
                .execute()) {
            if (httpResponse == null) {
                return AjaxResult.error("【queryPlanBusinessInfoUrl】请求返回结果 httpResponse 为空");
            }
            int code = httpResponse.getStatus();
            log.info("【queryPlanBusinessInfo】本次请求status：{}，地址：{}，返回结果是：{}", code, queryPlanBusinessInfoUrl, httpResponse.body());
            if (httpResponse.isOk()) {
                String jsonStr = httpResponse.body();
                JSONObject jsonObject = JSONObject.parseObject(jsonStr);
                if (StringUtils.isNotBlank(jsonObject.getString("code"))) {
                    return AjaxResult.error("queryPlanBusinessInfoUrl获取失败：返回数据：" + jsonStr);
                }
                return AjaxResult.success("queryPlanBusinessInfoUrl获取成功", jsonStr);
            } else {
                log.info("【queryPlanBusinessInfoUrl】请求数据失败，请求地址：{}，请求头部：{}，请求参数：{}，返回结果是：{}", queryPlanBusinessInfoUrl, headerMap, bodyMap, httpResponse.body());
            }
            return AjaxResult.error(httpResponse.body());
        } catch (HttpException e) {
            log.error("", e);
            log.info("【queryPlanBusinessInfoUrl】请求异常，请求地址：{}，请求头部：{}，请求参数：{}，异常信息：{}", queryPlanBusinessInfoUrl, headerMap, bodyMap, e.getMessage());
            return AjaxResult.error(e.getMessage());
        }
    }

    /**
     * 通过生产计划id获取详情
     *
     * @param proPlanId
     * @return
     */
    public AjaxResult getProducePlanDetailByApi(String proPlanId, String bureauCode, int timeout) {
        if (StringUtils.isBlank(bureauCode)) bureauCode = "0301";
        Map<String, Object> bodyMap = new HashMap<>();
        bodyMap.put("prodPlanIds", proPlanId);
        bodyMap.put("bureauCode", bureauCode);
        bodyMap.put("serviceSource", 29);
        AjaxResult getTokenResult = getToken(timeout);
        String token = StringUtils.EMPTY;
        //无论token获取成功或失败，都进行详情获取
        if (getTokenResult.isSuccess()) {
            token = Utils.getString(getTokenResult.get(AjaxResult.DATA_TAG));
        }
        return queryPlanInfoForPowerSupply(bodyMap, timeout, token);
    }

    /**
     * 接口查询生产计划工单信息
     *
     * @param bodyMap
     * @param timeout
     * @param token
     * @return
     */
    public AjaxResult queryPlanInfoForPowerSupply(Map<String, Object> bodyMap, int timeout, String token) {
        Map<String, String> headerMap = new HashMap<>();
        headerMap.put("Content-Type", "application/json");
        headerMap.put("access-token", token);
        headerMap.put("topProvinceFlag", TOP_PROVINCE_FLAG);
        log.info("【queryPlanInfoForPowerSupply】开始请求，地址：{}，请求头：{}，请求体：{}", queryPlanInfoForPowerSupplyUrl, headerMap, bodyMap);
        try (HttpResponse httpResponse = HttpRequest.post(queryPlanInfoForPowerSupplyUrl)
                .headerMap(headerMap, false)
                .timeout(timeout)
                .body(JSONObject.toJSONString(bodyMap))
                .execute()) {
            if (httpResponse == null) {
                return AjaxResult.error("【queryPlanInfoForPowerSupply】请求返回结果 httpResponse 为空");
            }
            int code = httpResponse.getStatus();
            log.info("【queryPlanInfoForPowerSupply】本次请求status：{}，地址：{}，返回结果是：{}", code, queryPlanInfoForPowerSupplyUrl, httpResponse.body());
            if (httpResponse.isOk()) {
                String jsonStr = httpResponse.body();
                return AjaxResult.success("queryPlanInfoForPowerSupply获取成功", jsonStr);
            } else {
                log.info("【queryPlanInfoForPowerSupply】请求数据失败，请求地址：{}，请求头部：{}，请求参数：{}，返回结果是：{}", queryPlanInfoForPowerSupplyUrl, headerMap, bodyMap, httpResponse.body());
            }
            return AjaxResult.error(httpResponse.body());
        } catch (HttpException e) {
            log.error("", e);
            log.info("【queryPlanInfoForPowerSupply】请求异常，请求地址：{}，请求头部：{}，请求参数：{}，异常信息：{}", queryPlanInfoForPowerSupplyUrl, headerMap, bodyMap, e.getMessage());
            return AjaxResult.error(e.getMessage());
        }
    }


    /**
     * 获取token
     *
     * @return
     */
    public AjaxResult getToken(int timeout) {
        Map<String, Object> bodyMap = new HashMap<>();
        bodyMap.put("account", getTokenAccount);
        bodyMap.put("password", getTokenPassword);
        bodyMap.put("authCode", getTokenAuthCode);
        log.info("【queryPlanInfoForPowerSupply】开始请求，地址：{}，请求体：{}", queryPlanInfoForPowerSupplyUrl, bodyMap);
        try (HttpResponse httpResponse = HttpRequest.post(getTokenUrl)
                .timeout(timeout)
                .header("Content-Type", "application/json")
                .body(JSONObject.toJSONString(bodyMap))
                .execute()) {

            if (httpResponse == null) {
                return AjaxResult.error("【getToken】请求返回结果 httpResponse 为空");
            }
            int code = httpResponse.getStatus();
            log.info("【getToken】本次请求status：{}，地址：{}，返回结果是：{}", code, getTokenUrl, httpResponse.body());
            if (!httpResponse.isOk()) {
                log.info("【getToken】请求数据失败，请求地址：{}，请求参数：{}，返回结果是：{}", getTokenUrl, bodyMap, httpResponse.body());
                return AjaxResult.error("【getToken】请求数据失败，返回结果是：" + httpResponse.body());
            }
            String jsonStr = httpResponse.body();
            JSONObject jsonObject = JSONObject.parseObject(jsonStr);
            if (jsonObject.getInteger("code") == 1) {
                String token = jsonObject.getString("token");
                return AjaxResult.success("获取token成功", token);
            } else {
                log.info("【getToken】请求返回结果异常，请求地址：{}，请求参数：{}，返回结果是：{}", getTokenUrl, bodyMap, httpResponse.body());
                return AjaxResult.error(httpResponse.body());
            }
        } catch (HttpException e) {
            log.error("", e);
            log.info("【getToken】请求异常，请求地址：{}，请求参数：{}，异常信息：{}", getTokenUrl, bodyMap, e.getMessage());
            return AjaxResult.error(e.getMessage());
        }
    }
}
