package com.sinoqixin.bgdrocketmq.service;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.sinoqixin.bgdrocketmq.dao.ProducePlanDao;
import com.sinoqixin.bgdrocketmq.service.handle.MqDataHandle;
import com.sinoqixin.bgdrocketmq.utils.DateUtil;
import com.sinoqixin.bgdrocketmq.utils.Utils;
import com.sinoqixin.bgdrocketmq.vo.AjaxResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class ProducePlanService implements MqDataHandle {

    //生成计划通知(生产工单推送)
    private final static String TOPIC = "gmp-sp-operationmaintenace-service-prodplan-plan-change";
    private final static String TAB = "*";
    //计划来源，164-广州智慧保供电，通过这个字段判断是否是保供电生成的计划
    private final static String PLAN_SOURCE_TYPE = "164";
    private final static int TIME_OUT = 30 * 1000; // 30s请求超时

    @Autowired
    private ProducePlanDao producePlanDao;
    @Value("${err.file.path}")
    private String errPath; //记录消费出错的数据路径
    @Autowired
    private GetProducePlanDetailApiService getProducePlanDetailApiService;

    @Override
    public void handleData(String msg) {
        try {
            List<Object[]> args = new ArrayList<>();
            List<Object[]> fullArgs = new ArrayList<>();
            List<Object[]> bgdArgs = new ArrayList<>();
            List<Object[]> planSiteArgs = new ArrayList<>();
            //设置参数
            setArgsBgdArgsData(msg, args, bgdArgs, fullArgs, planSiteArgs);
            if (!fullArgs.isEmpty()) {
                try {
                    log.info("更新 电网管理平台生产计划工单表【接口返回字段】，数据：{}", Utils.logListTypeIsObject(fullArgs));
                    recordErrDataToFile(fullArgs, "DWGL_FULL_NORMAL");
                    producePlanDao.updateDwglProducePlanWorkOrderFull(fullArgs);
                    producePlanDao.insertDwglProducePlanWorkOrderFullRecord(fullArgs);
                } catch (Exception e) {
                    recordErrDataToFile(fullArgs, "DWGL_ERR");
                    log.error("", e);
                    log.info("更新 电网管理平台生产计划工单表【接口返回字段】，异常信息：{}", e.getMessage());
                }
            }
            if (!args.isEmpty()) {
                try {
                    log.info("更新 电网管理平台生产计划工单表【rocketMq返回字段】，数据：{}", Utils.logListTypeIsObject(args));
                    recordErrDataToFile(args, "DWGL_NORMAL");
                    producePlanDao.updateDwglProducePlanWorkOrder(args);
                    producePlanDao.insertDwglProducePlanWorkOrderRecord(args);
                } catch (Exception e) {
                    recordErrDataToFile(args, "DWGL_ERR");
                    log.error("", e);
                    log.info("更新 电网管理平台生产计划工单表【rocketMq返回字段】，异常信息：{}", e.getMessage());
                }
            }
            if (!bgdArgs.isEmpty()) {
                try {
                    log.info("更新 保供电任务信息表，数据：{}", Utils.logListTypeIsObject(bgdArgs));
                    recordErrDataToFile(bgdArgs, "BGD_NORMAL");
                    producePlanDao.updateBgdTaskInfo(bgdArgs);
                } catch (Exception e) {
                    recordErrDataToFile(bgdArgs, "BGD_ERR");
                    log.error("", e);
                    log.info("更新 保供电任务信息表，异常信息：{}", e.getMessage());
                }
            }
        } catch (Exception e) {
            log.error("", e);
            log.info("生成计划通知(计划创建推送)数据解析出错，出错数据：{}，异常信息：{}", msg, e.getMessage());
        }
    }

    /**
     * 获取接口返回的字段数据
     *
     * @param jkItemObjectMap
     * @param businessObjectId
     * @return
     */
    public Object[] getQueryPlanInfoForPowerSupplyProp(Map<String, Object> jkItemObjectMap, String businessObjectId) {
        String planType = Utils.getString(jkItemObjectMap.get("planType"));
        String planNature = Utils.getString(jkItemObjectMap.get("planNature"));
        String workTeamOid = Utils.getString(jkItemObjectMap.get("workTeamOid"));
        String workTeamOcode = Utils.getString(jkItemObjectMap.get("workTeamOcode"));
        String workTeamOname = Utils.getString(jkItemObjectMap.get("workTeamOname"));
        String workSiteNames = Utils.getString(jkItemObjectMap.get("workSiteNames"));
        String working = Utils.getString(jkItemObjectMap.get("working"));
        String workSmallType = Utils.getString(jkItemObjectMap.get("workSmallType"));
        String creatorUid = Utils.getString(jkItemObjectMap.get("creatorUid"));
        String creatorUname = Utils.getString(jkItemObjectMap.get("creatorUname"));

        String createTime = Utils.getTimeFormatter(jkItemObjectMap.get("createTime"));
        String workMasterUid = Utils.getString(jkItemObjectMap.get("workMasterUid"));
        String workMasterUname = Utils.getString(jkItemObjectMap.get("workMasterUname"));
        String workMemberUnames = Utils.getString(jkItemObjectMap.get("workMemberUnames"));
        String realBeginTime = Utils.getTimeFormatter(jkItemObjectMap.get("realBeginTime"));
        String realEndTime = Utils.getTimeFormatter(jkItemObjectMap.get("realEndTime"));
        String baseVoltageId = Utils.getString(jkItemObjectMap.get("baseVoltageId"));
        String workDeptOid = Utils.getString(jkItemObjectMap.get("workDeptOid"));
        String workDeptOname = Utils.getString(jkItemObjectMap.get("workDeptOname"));
        String patrolMode = Utils.getString(jkItemObjectMap.get("patrolMode"));

        String provinceCode = Utils.getString(jkItemObjectMap.get("provinceCode"));
        String flowState = Utils.getString(jkItemObjectMap.get("flowState"));
        String monitorOpinion = Utils.getString(jkItemObjectMap.get("monitorOpinion"));
        String affirmUid = Utils.getString(jkItemObjectMap.get("affirmUid"));
        String affirmUname = Utils.getString(jkItemObjectMap.get("affirmUname"));
        String affirmTime = Utils.getTimeFormatter(jkItemObjectMap.get("affirmTime"));
        String isBequeathQuestion = Utils.getString(jkItemObjectMap.get("isBequeathQuestion"));
        String riskLevel = Utils.getString(jkItemObjectMap.get("riskLevel"));
        String notApplyOutage = Utils.getString(jkItemObjectMap.get("notApplyOutage"));
        String implementMode = Utils.getString(jkItemObjectMap.get("implementMode"));

        String updateTime = Utils.getTimeFormatter(jkItemObjectMap.get("updateTime"));
        String dataFrom = Utils.getString(jkItemObjectMap.get("dataFrom"));
        String startWorkTime = Utils.getTimeFormatter(jkItemObjectMap.get("startWorkTime"));
        String planTaskType = Utils.getString(jkItemObjectMap.get("planTaskType"));
        String jobContent = Utils.getString(jkItemObjectMap.get("jobContent"));
        String workResult = Utils.getString(jkItemObjectMap.get("workResult"));
        String bequeathQuestion = Utils.getString(jkItemObjectMap.get("bequeathQuestion"));
        String planSiteLst = Utils.getString(jkItemObjectMap.get("planSiteLst"));


        String prodPlanId = Utils.getString(jkItemObjectMap.get("prodPlanId"));
        String planCode = Utils.getString(jkItemObjectMap.get("planCode"));
        String planState = Utils.getString(jkItemObjectMap.get("planState"));
        String speciality = Utils.getString(jkItemObjectMap.get("speciality"));
        String bureauCode = Utils.getString(jkItemObjectMap.get("bureauCode"));
        String powerGridFlag = Utils.getString(jkItemObjectMap.get("powerGridFlag"));
        String workType = Utils.getString(jkItemObjectMap.get("workType"));

        String planBeginTime = Utils.getTimeFormatter(jkItemObjectMap.get("planBeginTime"));
        String planEndTime = Utils.getTimeFormatter(jkItemObjectMap.get("planEndTime"));

        String id = Utils.getString(jkItemObjectMap.get("id"));
        String businessType = Utils.getString(jkItemObjectMap.get("businessType"));
        String patrolClassify = Utils.getString(jkItemObjectMap.get("patrolClassify"));

        Object[] arg = {prodPlanId, planCode, planState, speciality, bureauCode, powerGridFlag, workType, planBeginTime, planEndTime,
                id, businessType, patrolClassify};

        Object[] subArg = {planType, planNature, workTeamOid, workTeamOcode, workTeamOname, workSiteNames, working,
                workSmallType, creatorUid, creatorUname, createTime, workMasterUid, workMasterUname, workMemberUnames,
                realBeginTime, realEndTime, baseVoltageId, workDeptOid, workDeptOname, patrolMode, provinceCode,
                flowState, monitorOpinion, affirmUid, affirmUname, affirmTime, isBequeathQuestion, riskLevel, notApplyOutage,
                implementMode, updateTime, dataFrom, startWorkTime, planTaskType, jobContent, workResult, bequeathQuestion,
                planSiteLst, businessObjectId};

        Object[] resultArg = new Object[arg.length + subArg.length];
        System.arraycopy(arg, 0, resultArg, 0, arg.length);
        System.arraycopy(subArg, 0, resultArg, arg.length, subArg.length);

        Object[] resultArg2 = new Object[resultArg.length * 2];
        System.arraycopy(resultArg, 0, resultArg2, 0, resultArg.length);
        System.arraycopy(resultArg, 0, resultArg2, resultArg.length, resultArg.length);
        return resultArg2;
    }

    /**
     * 设置参数
     *
     * @param msg
     * @param args
     * @param bgdArgs
     */
    public void setArgsBgdArgsData(String msg, List<Object[]> args, List<Object[]> bgdArgs, List<Object[]> fullArgs, List<Object[]> planSiteArgs) {
        JSONObject jsonObject = JSONObject.parseObject(msg);
        String businessType = jsonObject.getString("businessType");
        String id = jsonObject.getString("id");
        JSONObject ext = jsonObject.getJSONObject("ext");
        JSONObject msgObject = ext.getJSONObject("msg");
        String planListStr = msgObject.getString("planList");
        List<Map<String, Object>> planList = JSONObject.parseObject(planListStr, new TypeReference<List<Map<String, Object>>>() {
        });
        for (Map<String, Object> plan : planList) {
            String planSourceType = Utils.getString(plan.get("planSourceType")).trim();
            if (!PLAN_SOURCE_TYPE.equals(planSourceType)) continue;

            String prodPlanId = Utils.getString(plan.get("prodPlanId")).trim();  //生产计划ID
            if (StringUtils.isBlank(prodPlanId)) continue;

            String planState = Utils.getString(plan.get("planState")).trim(); //计划状态(10:新增 40:已发布(未派工) 50:执行中 60:待确认 70:已完成110:待开工)
            String planCode = Utils.getString(plan.get("planCode")).trim(); //计划编号

            //专业(1:高压专业 2:化学专业 3:继保专业 4:自动化专业 5:通信专业 6:直流专业
            //7:电测专业 8:安自专业 9:热工仪表专业 10:变电运行 11:输电线路 12:输电电缆 13:带电专业 14:检修专业 15:配电专业 16:计量专业 17:站用电源 18：监测专业 22：网络安全专业（调度）)
            String speciality = Utils.getString(plan.get("speciality")).trim();

            String bureauCode = Utils.getString(plan.get("bureauCode")).trim(); //局编码
            String powerGridFlag = Utils.getString(plan.get("powerGridFlag")).trim();  //电网类型(1:主网 2:配网)
            String patrolClassify = Utils.getString(plan.get("patrolClassify")).trim();  //巡视分类(1:变电 2:输电 3:配电)

            String planBeginTime = Utils.getTimeFormatter(plan.get("planBeginTime"));
            String workType = Utils.getString(plan.get("workType")).trim();  //工作类别（3：巡视）
            String planEndTime = Utils.getTimeFormatter(plan.get("planEndTime"));

            String jkResult = StringUtils.EMPTY;

            Object[] arg = {prodPlanId, planCode, planState, speciality, bureauCode, powerGridFlag, workType, planBeginTime, planEndTime,
                    id, businessType, patrolClassify};

            List<Object[]> fullList = new ArrayList<>();

            AjaxResult planBusinessInfoByApi = getProducePlanDetailApiService.getPlanBusinessInfoByApi(prodPlanId, bureauCode, TIME_OUT);

            if (planBusinessInfoByApi.isSuccess()) {
                try {
                    jkResult = Utils.getString(planBusinessInfoByApi.get(AjaxResult.DATA_TAG));
                    JSONObject jkResultObj = JSONObject.parseObject(jkResult);
                    String planInfoListStr = jkResultObj.getString("planList");
                    List<Map<String, Object>> planInfoList = JSONObject.parseObject(planInfoListStr, new TypeReference<List<Map<String, Object>>>() {
                    });
                    if (!planInfoList.isEmpty()) {
                        for (Map<String, Object> jkItemObjectMap : planInfoList) {
                            String businessObjectId = Utils.getString(jkItemObjectMap.get("businessObjectId"));
                            String prodPlanItemId = Utils.getString(jkItemObjectMap.get("prodPlanId"));
                            String planItemCode = Utils.getString(jkItemObjectMap.get("planCode"));
                            String planItemState = Utils.getString(jkItemObjectMap.get("planState"));
                            String planSiteLst = Utils.getString(jkItemObjectMap.get("planSiteLst"));

                            Object[] fullArg = getQueryPlanInfoForPowerSupplyProp(jkItemObjectMap, businessObjectId);
                            fullList.add(fullArg);
                            Object[] bgdArg = {businessObjectId, prodPlanItemId, planItemCode, getPlanState(planItemState)};
                            bgdArgs.add(bgdArg);
                        }
                    }
                } catch (Exception e) {
                    log.error("", e);
                    log.info("【queryPlanBusinessInfo】请求返回结果成功，但解析失败，生产计划id：{}，返回结果：{}，失败原因：{}", prodPlanId, jkResult, e.getMessage());
                }
            }

            if (!fullList.isEmpty()) {
                fullArgs.addAll(fullList);
            } else {
                Object[] arg2 = new Object[arg.length * 2];
                System.arraycopy(arg, 0, arg2, 0, arg.length);
                System.arraycopy(arg, 0, arg2, arg.length, arg.length);
                args.add(arg2);
            }
        }
    }

    public void setPlanSiteArgs(String planSiteLst, List<Object[]> planSiteArgs) {
        if (StringUtils.isNotBlank(planSiteLst)) {
            List<Map<String, Object>> mapList = JSONObject.parseObject(planSiteLst, new TypeReference<List<Map<String, Object>>>() {
            });
        }
    }

    //保供电任务信息表：计划状态: 1-未开始，2-已生成，3-派工，4-归档，5-取消
    //电网管理平台生产计划工单表：
    //1.生成计划通知(计划创建推送) 10:新增
    //2.消息通知(派工推送) 110:待开工
    //3.消息通知(计划确认推送) 70:已完成
    //4.消息通知(计划取消推送) 90:已取消
    //计划状态(10:新增 20:审批中 30:待发布 40:已发布(未派工) 50:执行中 60:待确认 70:已完成 80:变更中 90:已取消 100:待月度发布 110:待开工)
    public String getPlanState(String planState) {
        String result = planState;
        if ("10".equals(planState)) {
            result = "2";
        } else if ("110".equals(planState) || "50".equals(planState) || "60".equals(planState)) {
            result = "3";
        } else if ("70".equals(planState)) {
            result = "4";
        } else if ("90".equals(planState)) {
            result = "5";
        }
        return result;
    }

    /**
     * 将报错数据记录到err文件中
     *
     * @param args
     * @param type
     */
    public void recordErrDataToFile(List<Object[]> args, String type) {
        String filePath = errPath + DateUtil.getCurrentDateStr() + "/" + TOPIC + "_" + type + "_KAFKA.txt";
        String msg = "\n" +
                "------------ " + DateUtil.getCurrDateTimeStr() + " ------------" +
                "\n" +
                Utils.logListTypeIsObject(args);
        Utils.writeToFile(filePath, msg, true, false);
    }

    @Override
    public String getTopicName() {
        return TOPIC;
    }

    @Override
    public String getTabName() {
        return TAB;
    }
}
