package com.sinoqixin.bgdrocketmq.utils;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Calendar;
import java.util.Date;

@Slf4j
public class DateUtil {

    public static final String YYYY_MM_DD = "yyyy-MM-dd";

    private static DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

    public static String getCurrDateTimeStr() {
        return LocalDateTime.now().format(formatter);
    }

    public static Date parseCommonStr(String dateStr) {
        return parseCommonStrByFormat(dateStr, "yyyy-MM-dd HH:mm:ss");
    }

    public static String getDateStrByLong(long time, String pattern) {
        try {
            if (pattern == null) pattern = "yyyy-MM-dd HH:mm:ss";
            SimpleDateFormat sdf = new SimpleDateFormat(pattern);
            Date date = new Date(time);
            return sdf.format(date);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("时间转换异常：{}", e.getMessage());
            return null;
        }
    }

    public static Date parseCommonStrByFormat(String dateStr, String format) {
        SimpleDateFormat dateFormat = new SimpleDateFormat(format);
        try {
            Date date = dateFormat.parse(dateStr);
            return date;
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static Date getNextDate(Date refenceDate, int intevalDays) {
        try {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(refenceDate);
            calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) + intevalDays);
            return calendar.getTime();
        } catch (Exception ee) {
            return null;
        }
    }

    public static String getStrDate(String dateStr) {
        SimpleDateFormat dateFormat1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat dateFormat2 = new SimpleDateFormat("yyyyMMdd");

        try {
            Date date = dateFormat1.parse(dateStr);
            return dateFormat2.format(date);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static LocalDateTime parseTimeStr(String dateTimeStr) {
        return LocalDateTime.parse(dateTimeStr, formatter);
    }

    public static String getNextHour(Date refenceDate, int intevalHour, String formatStr) {
        try {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(refenceDate);
            calendar.set(Calendar.HOUR, calendar.get(Calendar.HOUR) + intevalHour);
            SimpleDateFormat df = new SimpleDateFormat(formatStr);
            return df.format(calendar.getTime());
        } catch (Exception ee) {
            return "";
        }
    }

    public static String getTimeHourBefore(int hours) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY,
                calendar.get(Calendar.HOUR_OF_DAY) - hours);
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        return df.format(calendar.getTime());
    }

    public static String parseDateToStr(Date dateTime, String format) {
        String result = StringUtils.EMPTY;
        try {
            if (StringUtils.isBlank(format)) format = "yyyy-MM-dd HH:mm:ss";
            SimpleDateFormat sdf = new SimpleDateFormat(format);
            result = sdf.format(dateTime);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    public static String getCurrentDateStr() {
        /*Calendar cal = Calendar.getInstance();
        return  ""+cal.get(Calendar.YEAR)+cal.get(Calendar.MONTH)+cal.get(Calendar.DAY_OF_MONTH);*/
        SimpleDateFormat df = new SimpleDateFormat("yyyyMMdd");
        return df.format(new Date());
    }

    public static String getYesterdayDateStr() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DAY_OF_MONTH,
                calendar.get(Calendar.DAY_OF_MONTH) - 1);
        SimpleDateFormat df = new SimpleDateFormat("yyyyMMdd");

        return df.format(calendar.getTime());
    }

    public static int getHour(String str) throws ParseException {
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        Date date = df.parse(str);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        int hour = calendar.get(Calendar.HOUR_OF_DAY);
        return hour;
    }

    public static String getNextDateStr(String dateStr, int interval) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        try {
            Date date = sdf.parse(dateStr);
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(Calendar.DAY_OF_MONTH, calendar.get(Calendar.DAY_OF_MONTH) + interval);

            return sdf.format(calendar.getTime());
        } catch (ParseException e) {
            return null;
        }
    }

    /**
     * 获取当前时间往后推 interval 年日期
     *
     * @param interval
     * @return
     */
    public static String getNextYearStr(int interval) {
        return getNextYearStr(interval, "yyyy-MM-dd");
    }

    /**
     * 获取当前时间往后推 interval 年日期
     *
     * @param interval
     * @return
     */
    public static String getNextYearStr(int interval, String format) {
        Calendar instance = Calendar.getInstance();
        instance.add(Calendar.YEAR, interval);
        Date time = instance.getTime();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(format);
        return simpleDateFormat.format(time);
    }
}

