package com.sinoqixin.bgdrocketmq.utils;

import cn.hutool.core.io.FileUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;

import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

@Slf4j
public class Utils {

    public final static int KAFKA_HANDLE_TYPE_0 = 0;

    public final static int KAFKA_HANDLE_TYPE_1 = 1;

    /**
     * 只保留是广州编码的数据
     *
     * @param args
     * @param gzCode
     * @param index
     * @return
     */
    public static List<Object[]> getGzBureauCodeList(List<Object[]> args, String gzCode, int index) {
        List<Object[]> objects = new ArrayList<>();
        for (Object[] arg : args) {
            String bureauCode = Utils.getString(arg[index]).trim();
            if (gzCode.equals(bureauCode)) {
                objects.add(arg);
            }
        }
        return objects;
    }

    /**
     * 写入文件
     *
     * @param filePath
     * @param msg
     * @param isAppend
     * @param format
     */
    public static void writeToFile(String filePath, String msg, boolean isAppend, boolean format) {
        try {
            File file = new File(filePath);
            if (format) {
                msg = JSON.toJSONString(JSONObject.parseObject(msg), SerializerFeature.PrettyFormat,
                        SerializerFeature.WriteMapNullValue, SerializerFeature.WriteDateUseDateFormat);
            }
            if (!file.exists()) {
                if (!file.getParentFile().exists()) {
                    file.getParentFile().mkdirs();
                }
                file.createNewFile();
            }
            if (isAppend) {
                FileUtil.appendUtf8String(msg, file);
            } else {
                FileUtil.writeUtf8String(msg, file);
            }
        } catch (IOException e) {
            e.printStackTrace();
            log.info("写入文件异常，异常信息：{}", e.getMessage());
        }
    }

    /**
     * 将List<Object[]>转为string类型，用于打印输出。
     *
     * @param list
     * @return
     */
    public static String logListTypeIsObject(List<Object[]> list) {
        StringBuilder val = new StringBuilder();
        if (list == null || list.isEmpty()) return val.toString();
        for (Object[] objects : list) {
            val.append(Arrays.toString(objects)).append("\n");
        }
        return val.toString();
    }

    /**
     * 获取uuid
     *
     * @return
     */
    public static String getUUID() {
        UUID uuid = UUID.randomUUID();
        String uuidStr = uuid.toString();
        uuidStr = uuidStr.replace("-", "");
        return uuidStr;
    }

    /**
     * 将值转化成字符串
     *
     * @param num
     * @return
     */
    public static String getString(Object num) {
        return (num != null &&
                !"".equals(num) &&
                !"null".equals(num) &&
                !"undefined".equals(num) &&
                !"NULL".equals(num))
                ? String.valueOf(String.valueOf(num)) : "";
    }

    /**
     * 将值转化成数字
     *
     * @param num
     * @return
     */
    public static int getInteger(Object num) {
        String str = getString(num);
        return !"".equals(str) ? Integer.valueOf(str) : 0;
    }

    public static Long getLong(Object num) {
        String str = getString(num);
        return !"".equals(str) ? Long.valueOf(str) : 0L;
    }

    /**
     * 判断表是否存在
     *
     * @param tableName
     * @return
     */
    public static boolean getTableIsExist(JdbcTemplate jdbcTemplate, String patternName, String tableName) {
        String sql = "SELECT COUNT(*) COUNT FROM USER_TABLES  WHERE TABLE_NAME = ? ";
        List<Map<String, Object>> mapList = jdbcTemplate.queryForList(sql, tableName);
        if (!mapList.isEmpty()) {
            int count = Utils.getInteger(mapList.get(0).get("COUNT"));
            return count > 0;
        }
        return false;
    }

    public static String strToNull(Object num) {
        String str = getString(num).trim();
        return !"".equals(str) ? str : null;
    }

    /**
     * 打印错误SQL
     *
     * @param sql  preStatement
     * @param args args
     */
    public static void logErrorSql(String sql, List<Object[]> args) {
        for (Object[] arg : args) {
            String formattedSql = sql;
            for (Object parameter : arg) {
                formattedSql = formattedSql.replaceFirst("\\?", parameter != null ? String.format("'%s'", parameter) : "NULL");
            }
            log.error("Executing SQL: {}", formattedSql);
        }
    }

    /**
     * 时间戳字符转时间格式字符
     *
     * @param timeObj
     * @return
     */
    public static String getTimeFormatter(Object timeObj) {
        String timeStr = getString(timeObj);
        String result = StringUtils.EMPTY;
        try {
            if (StringUtils.isBlank(timeStr)) return result;
            Long time = Utils.getLong(timeStr);
            DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date date = new Date(time);
            result = dateFormat.format(date);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("日期转换异常，异常信息：{}", e.getMessage());
        }
        return result;
    }

    /**
     * 删除某目录下，以日期命名的文件夹。可设置删除某天前的文件夹。
     *
     * @param directoryPath
     * @param dayNum
     */
    public static void deleteDirectoryByDayNum(String directoryPath, int dayNum) {
        try {
            File file = new File(directoryPath);
            File[] files = file.listFiles();
            String currentDateStr = DateUtil.getCurrentDateStr();
            String day7Before = DateUtil.getNextDateStr(currentDateStr, dayNum);
            for (File fileItem : files) {
                if (!fileItem.isDirectory()) continue;
                String fileName = fileItem.getName();
                if (fileName.length() != 8) continue;
                if (day7Before == null) continue;
                if (fileName.compareTo(day7Before) < 0) {
                    FileUtils.deleteDirectory(fileItem);
                    log.info("删除文件夹：{}", fileName);
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
